/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#if defined(STR_DEBUG) || defined(STR_CKSUFFIX_DEBUG)
#include <stdio.h>
#endif
#include <stddef.h>
#include <string.h>
#include "ulstring.h"

/* Check file suffix/extension, return true if src has suffix suf
 * false otherwise. */
bool cksuffix(const char *src, const char *suf)
{
	size_t ls = strlen(suf);
	char *p = strrstr(src, suf);
#ifdef STR_DEBUG
	printf("cksuffix(): src=\"%s\", strlen(suf)=%u, suf=\"%s\"\n", src, (unsigned)ls, suf);
#endif
	if(ls == 0 || !p) {
#ifdef STR_DEBUG
		printf("cksuffix(): sanity check failed or suf not found in src\n");
#endif
		goto end;
	}
#ifdef STR_DEBUG
	printf("cksuffix(): suf found in src, p=\"%s\"\n", p);
#endif
	if(*(p-1) == '.' && *(p+ls) == '\0') {
#ifdef STR_DEBUG
		printf("cksuffix(): returning true\n");
#endif
		return true;
	}
end:
#ifdef STR_DEBUG
	printf("cksuffix(): returning false\n");
#endif
	return false;
}

#ifdef STR_CKSUFFIX_DEBUG
int main(void)
{
	cksuffix("", "");
	cksuffix("lasdjflkasjf", "");
	cksuffix("", "asd");
	cksuffix("/tmp/lasdjflkasjf/a.s", "s");
	cksuffix("lasdjflkasjf/a.s.s.s.s", "s");
	cksuffix("lasdjflkasjf/a.s.c.d.s.e", "s");
	cksuffix("/etc/vsftpd/vsftpd.conf", "conf");
	cksuffix("lasdjflkasj.f", "f");
	cksuffix("d:\\dos\\lasdjflkasjf\\a.exe", "exe");
	cksuffix("lasdjflkasjf", "lasdjflkasjf");
	cksuffix("lasdjflkasjf", "lasdjflkasjfaskldfj");
	return 0;
}
#endif
