/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/* string.h extension.
 */
#ifndef _ULSTRING_H
#define _ULSTRING_H

#include <stddef.h>
#include "ulbool.h"
#include "uldef.h"
_BEGIN_DECLS

/* Check for two or more identical chars in src,
 * return true on found, false on not found or empty string. */
bool ckidechr(const char *src);

/* malloc() n bytes and memcpy(dst, src, n); 
 * return the pointer or NULL on failed malloc(). */
void *memdup(void *src, size_t n);

/* Remove leading space(s) from src, 
 * return a pointer to changed src. */
char *rmlspc(const char *src);

/* Remove paratactic char(s) from src, leaving only one instance.
 * return a pointer to changed src. */
char *rmpchr(char *src, int c);

/* Remove trailing space(s) from src, 
 * return a pointer to changed src. */
char *rmtspc(char *src);

/* Count times of sub occurrence(s) in src. */
size_t strcstr(const char *src, const char *sub);

/* Strcat() with strlen(dst) returned. */
size_t strldcat(char *dst, const char *src);

/* Strcpy() with strlen(src) returned. */
size_t strldcpy(char *dst, const char *src);

/* Strcat() with strlen(src) returned. */
size_t strlscat(char *dst, const char *src);

/* Replace all matching sub(s) with repl in src, 
 * save result in a newly malloc()ed area,
 * reture a pointer to the area or NULL if malloc() failed.
 * src remain unchanged, don't forget to free()! 
 *
 * fast on large input (O(n)) if realloc() takes constant time, 
 * better choice when strlen(repl) > strlen(sub) many times. */
char *strrepl(const char *src, const char *sub, const char *repl);

/* Place src in reversed order, return a pointer to changed src. */
char *strrev(char *src);

/* Reversed strstr(), return a pointer to the beginning of 
 * the last matching sub, or NULL if not found. */
char *strrstr(const char *src, const char *sub);

/* Substitute all matching sub(s) with repl in src, 
 * src MUST be big enough to hold all changes!
 * return a pointer to changed src. if flag is true,
 * do it recursively (changed src becomes input again).
 *
 * slow (O(n square)) on large input, but generally 
 * OK and convenient when strlen(repl) <= strlen(sub). */
char *strsstr(char *src, const char *sub, const char *repl, bool flag);

_END_DECLS
#endif
